@@ -9,7 +9,6 @@ import (
 	"github.com/lmorg/murex/builtins/pipes/term"
 	"github.com/lmorg/murex/lang"
 	"github.com/lmorg/murex/lang/ref"
-	"github.com/lmorg/murex/lang/tty"
 	"github.com/lmorg/murex/shell/autocomplete"
 	"github.com/lmorg/murex/utils"
 	"github.com/lmorg/murex/utils/ansi"
@@ -40,33 +39,33 @@ func Execute(flags int) {
 
 	pwd, err := os.Getwd()
 	if err != nil {
-		tty.Stderr.WriteString(err.Error())
+		os.Stderr.WriteString(err.Error())
 	}
 
 	if flags&F_PRELOAD != 0 {
 		if err := profile(preloadFileName, PreloadPath()); err != nil {
-			tty.Stderr.WriteString("There were problems loading profile `" + PreloadPath() + "`:" + utils.NewLineString)
-			tty.Stderr.WriteString(err.Error() + utils.NewLineString)
+			os.Stderr.WriteString("There were problems loading profile `" + PreloadPath() + "`:" + utils.NewLineString)
+			os.Stderr.WriteString(err.Error() + utils.NewLineString)
 		}
 	}
 
 	if flags&F_MODULES != 0 {
 		if err := modules(ModulePath()); err != nil {
-			tty.Stderr.WriteString("There were problems loading modules `" + ModulePath() + "`:" + utils.NewLineString)
-			tty.Stderr.WriteString(err.Error() + utils.NewLineString)
+			os.Stderr.WriteString("There were problems loading modules `" + ModulePath() + "`:" + utils.NewLineString)
+			os.Stderr.WriteString(err.Error() + utils.NewLineString)
 		}
 	}
 
 	if flags&F_PROFILE != 0 {
 		if err := profile(profileFileName, ProfilePath()); err != nil {
-			tty.Stderr.WriteString("There were problems loading profile `" + ProfilePath() + "`:" + utils.NewLineString)
-			tty.Stderr.WriteString(err.Error() + utils.NewLineString)
+			os.Stderr.WriteString("There were problems loading profile `" + ProfilePath() + "`:" + utils.NewLineString)
+			os.Stderr.WriteString(err.Error() + utils.NewLineString)
 		}
 	}
 
 	err = os.Chdir(pwd)
 	if err != nil {
-		tty.Stderr.WriteString(err.Error())
+		os.Stderr.WriteString(err.Error())
 	}
 
 	lang.ShellProcess.FileRef = ref.NewModule(app.ShellModule)
@@ -102,12 +101,12 @@ func profile(name, path string) error {
 
 	err = os.Chdir(home.MyDir)
 	if err != nil {
-		tty.Stderr.WriteString(err.Error())
+		os.Stderr.WriteString(err.Error())
 	}
 
 	block := []rune(string(b))
 
-	tty.Stderr.WriteString("Loading profile `" + name + "`" + utils.NewLineString)
+	os.Stderr.WriteString("Loading profile `" + name + "`" + utils.NewLineString)
 
 	// lets redirect all output to STDERR just in case this thing gets piped for any strange reason
 	fork := lang.ShellProcess.Fork(lang.F_NEW_MODULE | lang.F_NEW_TESTS | lang.F_NO_STDIN)
