@@ -5,6 +5,7 @@ package main
 
 import (
 	"os"
+	"time"
 
 	"github.com/lmorg/murex/app/whatsnew"
 	_ "github.com/lmorg/murex/builtins"
@@ -13,7 +14,7 @@ import (
 	"github.com/lmorg/murex/config/profile"
 	"github.com/lmorg/murex/debug"
 	"github.com/lmorg/murex/lang"
-	"github.com/lmorg/murex/lang/tty"
+	"github.com/lmorg/murex/lang/ref"
 	"github.com/lmorg/murex/shell"
 	"github.com/lmorg/murex/utils/readline"
 )
@@ -62,7 +63,7 @@ func runTests() error {
 	if err := lang.ShellProcess.Config.Set("test", "verbose", false, nil); err != nil {
 		return err
 	}
-	tty := readline.IsTerminal(int(tty.Stdout.Fd()))
+	tty := readline.IsTerminal(int(os.Stdout.Fd()))
 	if err := lang.ShellProcess.Config.Set("shell", "color", tty, nil); err != nil {
 		return err
 	}
@@ -95,7 +96,12 @@ func runCommandLine(commandLine string) {
 
 	// read block from command line parameters
 	term.OutSetDataTypeIPC()
-	execSource([]rune(commandLine), nil, true)
+	sourceRef := ref.Source{
+		DateTime: time.Now(),
+		Filename: "",
+		Module:   "murex/-c",
+	}
+	execSource([]rune(commandLine), &sourceRef, true)
 
 	if fInteractive {
 		shell.Start()
@@ -121,7 +127,7 @@ func runSource(filename string) {
 	term.OutSetDataTypeIPC()
 	disk, err := diskSource(filename)
 	if err != nil {
-		_, err := tty.Stderr.WriteString(err.Error() + "\n")
+		_, err := os.Stderr.WriteString(err.Error() + "\n")
 		if err != nil {
 			// wouldn't really make any difference at this point because we
 			// cannot write to stderr anyway :(
@@ -129,14 +135,16 @@ func runSource(filename string) {
 		}
 		lang.Exit(1)
 	}
-	execSource([]rune(string(disk)), nil, true)
+
+	sourceRef := ref.Source{
+		DateTime: time.Now(),
+		Filename: filename,
+		Module:   "murex/#!",
+	}
+	execSource([]rune(string(disk)), &sourceRef, true)
 }
 
 func startMurex() {
-	if os.Getenv("MUREX_EXPERIMENTAL") != "" {
-		tty.CreatePTY()
-	}
-
 	lang.InitEnv()
 
 	// default config
