@@ -4,10 +4,9 @@
 package shell
 
 import (
+	"os"
 	"os/signal"
 	"syscall"
-
-	"github.com/lmorg/murex/lang/tty"
 )
 
 // Handler is an internal function to capture and handle OS signals (eg SIGTERM).
@@ -29,7 +28,7 @@ func SignalHandler(interactive bool) {
 				sigquit(interactive)
 
 			default:
-				tty.Stderr.WriteString("Unhandled signal: " + sig.String())
+				os.Stderr.WriteString("Unhandled signal: " + sig.String())
 			}
 		}
 	}()
